using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DgvFilterPopup;
using Microsoft.Office.Interop;
using MigraDoc.DocumentObjectModel;
using MigraDoc.Rendering;

public partial class BaseForm : Form
{
    public BaseForm()
    {
        InitializeComponent();
    }

    private bool bAddMode = false;
    private bool bEditMode = false;
    private bool bDeleteMode = false;
    private int iRow = 0;

    public bool AddMode
    {
        get { return bAddMode; }
        set { bAddMode = value; }
    }

    public bool EditMode
    {
        get { return bEditMode; }
        set { bEditMode = value; }
    }

    public bool DeleteMode
    {
        get { return bDeleteMode; }
        set { bDeleteMode = value; }
    }

    public int Row
    {
        get { return iRow; }
        set { iRow = value; }
    }

    public void BaseForm_Shown(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        DgvFilterManager filterManager = new DgvFilterManager(gridCode);
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    protected virtual void LoadGridCode()
    {
    }

    public void gridCode_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            try
            {
                Row = gridCode.CurrentRow.Index;
            }
            catch { }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }


    public void gridCode_DoubleClick(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            try
            {
                if (gridCode.Rows.Count > 0)
                {
                    ShowHideTabPages(TPForm, TPGrid);
                    Row = gridCode.CurrentRow.Index;
                    Edit();
                }
            }
            catch { }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    protected virtual void Add()
    {
    }

    protected virtual void Edit()
    {
    }

    protected virtual void Delete()
    {
    }

    protected virtual void EnableRecord(bool YesNo)
    {
    }

    protected virtual void ClearRecord()
    {
    }

    public void GoBack_To_Grid()
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        bool gridOK = false;
        try
        {
            LoadGridCode();
            ShowToolStripItems("Cancel");
            gridCode.Rows[Row].Selected = true;
            gridCode.FirstDisplayedScrollingRowIndex = Row;

            gridOK = true;
        }
        catch
        {
            //Hide error message.
        }
        finally
        {
            if (gridOK == false)
            {
                ShowToolStripItems("Cancel");
            }
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void butCancel_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        ShowToolStripItems("Cancel");
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void butClose_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        this.Close();
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void butApply_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        if (this.AddMode == true)
        {
            this.InsertRecord();
        }
        else if (this.EditMode == true)
        {
            this.UpdateRecord();
        }
        else if (this.DeleteMode == true)
        {
            this.DeleteRecord();
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    protected virtual void InsertRecord()
    {
    }

    protected virtual void UpdateRecord()
    {
    }

    protected virtual void DeleteRecord()
    {
    }

    protected virtual Boolean VerifyData()
    {
        return true;
    }

    public void ShowToolStripItems(string Item)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        bAdd.Enabled = false;
        bEdit.Enabled = false;
        bDelete.Enabled = false;
        bCancel.Enabled = false;
        bRefresh.Enabled = false;
        switch (Item)
        {
            case "Add":
                bCancel.Enabled = true;
                break;
            case "Edit":
                bCancel.Enabled = true;
                break;
            case "Delete":
                bCancel.Enabled = true;
                break;
            case "Cancel":
                if (gridCode.Rows.Count == 0)
                {
                    ShowToolStripItems("No Record");
                }
                else
                {
                    bAdd.Enabled = true;
                    bEdit.Enabled = true;
                    bDelete.Enabled = true;
                    bRefresh.Enabled = true;
                    ShowHideTabPages(TPGrid, TPForm);
                }
                break;
            case "Refresh":
                bAdd.Enabled = true;
                bEdit.Enabled = true;
                bDelete.Enabled = true;
                bRefresh.Enabled = true;
                LoadGridCode();
                break;
            case "No Record":
                bAdd.Enabled = true;
                ShowHideTabPages(TPGrid, TPForm);
                break;
        }
        TPForm.Text = Item.Trim();
        TPGrid.Text = "Categories";
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void ShowHideTabPages(TabPage Show, TabPage Hide)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        //if (this.TC.TabPages.Contains(Hide)) {
        //    this.TC.TabPages.Remove(Hide);}
        //if (this.TC.TabPages.Contains(Show)) {
        //    this.TC.TabPages.Remove(Show);}
        //if (!this.TC.TabPages.Contains(Show)) {
        //    this.TC.TabPages.Add(Show);}

        TC.ItemSize = new Size(0, 1);
        TC.Appearance = TabAppearance.FlatButtons;
        TC.SelectedTab = Show;
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void TS_ItemClicked(object sender, System.Windows.Forms.ToolStripItemClickedEventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        if (e.ClickedItem.Text != "Export")
        {
            ShowToolStripItems(e.ClickedItem.Text);
            TPForm.Text = e.ClickedItem.Text.Trim();
        }
        switch (e.ClickedItem.Text)
        {
            case "Add":
                ShowHideTabPages(TPForm, TPGrid);
                Add();
                break;
            case "Edit":
                ShowHideTabPages(TPForm, TPGrid);
                Edit();
                break;
            case "Delete":
                ShowHideTabPages(TPForm, TPGrid);
                Delete();
                break;
            case "Cancel":
                ShowToolStripItems("Cancel");
                break;
            case "Refresh":
                GoBack_To_Grid();
                break;
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void bWord_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        if (TC.SelectedTab == TPForm)
        {
            ExportToWord("One", this.Text, gridCode);
        }
        else if (TC.SelectedTab == TPGrid)
        {
            ExportToWord("Many", this.Text, gridCode);
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void bExcel_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        if (TC.SelectedTab == TPForm)
        {
            ExportToExcel("One", this.Text, gridCode);
        }
        else if (TC.SelectedTab == TPGrid)
        {
            ExportToExcel("Many", this.Text, gridCode);
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void bPDF_Click(object sender, System.EventArgs e)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        if (TC.SelectedTab == TPForm)
        {
            ExportToPDF("One", this.Text, gridCode, gridCode.CurrentRow);
        }
        else if (TC.SelectedTab == TPGrid)
        {
            ExportToPDF("Many", this.Text, gridCode, gridCode.CurrentRow);
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void ExportToExcel(string Type, string Head, System.Windows.Forms.DataGridView Grid)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        try
        {
            if (Grid.ColumnCount > 0)
            {
                Microsoft.Office.Interop.Excel.Application oExcel;
                Microsoft.Office.Interop.Excel.Workbook oBook;
                Microsoft.Office.Interop.Excel.Worksheet oSheet;

                oExcel = new Microsoft.Office.Interop.Excel.Application();
                oExcel.Visible = false;

                oBook = (Microsoft.Office.Interop.Excel.Workbook)(oExcel.Workbooks.Add(System.Type.Missing));
                oSheet = (Microsoft.Office.Interop.Excel.Worksheet)oBook.ActiveSheet;

                oSheet.Range["B2"].Value = Head;
                oSheet.Range["B2"].Font.Bold = true;

                if (Type == "One")
                {
                    for (int s = 0; s <= Grid.ColumnCount - 1; s++)
                    {
                        oSheet.Range["B" + (4 + s)].Value = Grid.Columns[s].HeaderText;
                        oSheet.Range["C" + (4 + s)].Value = Grid.SelectedRows[0].Cells[s].Value;
                    }

                    oSheet.Columns["B:B"].ColumnWidth = 20;
                    oSheet.Columns.AutoFit();

                }
                else if (Type == "Many")
                {
                    object[,] DataArrayHead = new object[1, Grid.ColumnCount];
                    for (int s = 0; s <= Grid.ColumnCount - 1; s++)
                    {
                        DataArrayHead[0, s] = Grid.Columns[s].HeaderText;
                    }

                    oSheet.Range["B4"].Resize[1, Grid.ColumnCount].Value = DataArrayHead;
                    oSheet.Range["B4"].Resize[1, Grid.ColumnCount].Font.Bold = true;

                    object[,] DataArray = new object[Grid.RowCount, Grid.ColumnCount];
                    for (int r = 0; r <= Grid.RowCount - 1; r++)
                    {
                        for (int s = 0; s <= Grid.ColumnCount - 1; s++)
                        {
                            DataArray[r, s] = Grid[s, r].Value;
                        }
                    }

                    oSheet.Range["B5"].Resize[Grid.RowCount, Grid.ColumnCount].Value = DataArray;
                    oSheet.Columns.AutoFit();

                }

                oExcel.Visible = true;
                oExcel = null;
                oSheet = null;
            }
        }
        catch
        {
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    public void ExportToWord(string Type, string Head, System.Windows.Forms.DataGridView Grid)
    {
        this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
        try
        {
            if (Grid.ColumnCount > 0)
            {
                Microsoft.Office.Interop.Word.Application oWord;
                Microsoft.Office.Interop.Word.Document oDoc;
                Microsoft.Office.Interop.Word.Table oTable;
                Microsoft.Office.Interop.Word.Paragraph oPara1;

                oWord = new Microsoft.Office.Interop.Word.Application();
                oWord.Visible = false;
                oDoc = (Microsoft.Office.Interop.Word.Document)(oWord.Documents.Add(System.Type.Missing));
                var _with1 = oDoc.PageSetup;
                _with1.PaperSize = Microsoft.Office.Interop.Word.WdPaperSize.wdPaperA4;
                _with1.Orientation = Microsoft.Office.Interop.Word.WdOrientation.wdOrientLandscape;
                _with1.LeftMargin = oWord.CentimetersToPoints(1);
                _with1.BottomMargin = oWord.CentimetersToPoints(1);
                _with1.TopMargin = oWord.CentimetersToPoints(1);

                oPara1 = (Microsoft.Office.Interop.Word.Paragraph)(oDoc.Content.Paragraphs.Add(System.Type.Missing));
                oPara1.Range.Text = Head;
                oPara1.Range.Font.Bold = 0;
                oPara1.Range.Font.Size = 8;
                oPara1.Format.SpaceAfter = 8;
                oPara1.Range.InsertParagraphAfter();

                if (Type == "One")
                {
                    oTable = oDoc.Tables.Add(oDoc.Bookmarks.get_Item("\\endofdoc").Range, Grid.ColumnCount, 2);
                    oTable.Range.ParagraphFormat.SpaceAfter = 2;

                    for (int i = 0; i <= Grid.ColumnCount - 1; i++)
                    {
                        oTable.Cell(i + 1, 1).Range.InsertAfter(Grid.Columns[i].HeaderText);
                        oTable.Cell(i + 1, 2).Range.Text = Convert.ToString(Grid.SelectedRows[0].Cells[i].Value);
                    }

                    oTable.Borders.InsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleSingle;
                    oTable.Borders.OutsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleDouble;

                    oTable.AllowAutoFit = true;
                    oTable.Columns.AutoFit();

                }
                else if (Type == "Many")
                {
                    oTable = oDoc.Tables.Add(oDoc.Bookmarks.get_Item("\\endofdoc").Range, Grid.RowCount + 1, Grid.ColumnCount);
                    oTable.Range.ParagraphFormat.SpaceAfter = 2;

                    for (int i = 0; i <= Grid.ColumnCount - 1; i++)
                    {
                        oTable.Cell(1, i + 1).Range.InsertAfter(Grid.Columns[i].HeaderText);
                    }

                    for (int r = 0; r <= Grid.RowCount - 1; r++)
                    {
                        for (int s = 0; s <= Grid.ColumnCount - 1; s++)
                        {
                            oTable.Cell(r + 2, s + 1).Range.Text = Convert.ToString(Grid[s, r].Value);
                        }
                    }

                    oTable.Rows[1].Cells.Shading.BackgroundPatternColorIndex = Microsoft.Office.Interop.Word.WdColorIndex.wdGray25;

                    oTable.Borders.InsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleSingle;
                    oTable.Borders.OutsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleDouble;

                    oTable.AllowAutoFit = true;
                    oTable.Columns.AutoFit();

                }

                oWord.Visible = true;
                oWord.NormalTemplate.Saved = true;
                oWord = null;
                oDoc = null;
            }
        }
        catch
        {
        }
        this.Cursor = System.Windows.Forms.Cursors.Default;
    }

    //public void ExportToPDF(string Type, string Head, System.Windows.Forms.DataGridView Grid)
    //{
    //    try
    //    {
    //        if (Grid.ColumnCount > 0)
    //        {
    //            Microsoft.Office.Interop.Word.Application oWord;
    //            Microsoft.Office.Interop.Word.Document oDoc;
    //            Microsoft.Office.Interop.Word.Table oTable;
    //            Microsoft.Office.Interop.Word.Paragraph oPara1;

    //            oWord = new Microsoft.Office.Interop.Word.Application();
    //            oWord.Visible = false;
    //            oDoc = (Microsoft.Office.Interop.Word.Document)(oWord.Documents.Add(System.Type.Missing));
    //            var _with1 = oDoc.PageSetup;
    //            _with1.PaperSize = Microsoft.Office.Interop.Word.WdPaperSize.wdPaperA4;
    //            _with1.Orientation = Microsoft.Office.Interop.Word.WdOrientation.wdOrientLandscape;
    //            _with1.LeftMargin = oWord.CentimetersToPoints(1);
    //            _with1.BottomMargin = oWord.CentimetersToPoints(1);
    //            _with1.TopMargin = oWord.CentimetersToPoints(1);

    //            oPara1 = (Microsoft.Office.Interop.Word.Paragraph)(oDoc.Content.Paragraphs.Add(System.Type.Missing));
    //            oPara1.Range.Text = Head;
    //            oPara1.Range.Font.Bold = 0;
    //            oPara1.Range.Font.Size = 8;
    //            oPara1.Format.SpaceAfter = 8;
    //            oPara1.Range.InsertParagraphAfter();


    //            if (Type == "One")
    //            {
    //                oTable = oDoc.Tables.Add(oDoc.Bookmarks.get_Item("\\endofdoc").Range, Grid.ColumnCount, 2);
    //                oTable.Range.ParagraphFormat.SpaceAfter = 2;

    //                for (int i = 0; i <= Grid.ColumnCount - 1; i++)
    //                {
    //                    oTable.Cell(i + 1, 1).Range.InsertAfter(Grid.Columns[i].HeaderText);
    //                    oTable.Cell(i + 1, 2).Range.Text = Convert.ToString(Grid.SelectedRows[0].Cells[i].Value);
    //                }

    //                oTable.Borders.InsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleSingle;
    //                oTable.Borders.OutsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleDouble;

    //                oTable.AllowAutoFit = true;
    //                oTable.Columns.AutoFit();

    //            }
    //            else if (Type == "Many")
    //            {
    //                oTable = oDoc.Tables.Add(oDoc.Bookmarks.get_Item("\\endofdoc").Range, Grid.RowCount + 1, Grid.ColumnCount);
    //                oTable.Range.ParagraphFormat.SpaceAfter = 2;

    //                for (int i = 0; i <= Grid.ColumnCount - 1; i++)
    //                {
    //                    oTable.Cell(1, i + 1).Range.InsertAfter(Grid.Columns[i].HeaderText);
    //                }

    //                for (int r = 0; r <= Grid.RowCount - 1; r++)
    //                {
    //                    for (int s = 0; s <= Grid.ColumnCount - 1; s++)
    //                    {
    //                        oTable.Cell(r + 2, s + 1).Range.Text = Convert.ToString(Grid[s, r].Value);
    //                    }
    //                }

    //                oTable.Rows[1].Cells.Shading.BackgroundPatternColorIndex = Microsoft.Office.Interop.Word.WdColorIndex.wdGray25;

    //                oTable.Borders.InsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleSingle;
    //                oTable.Borders.OutsideLineStyle = Microsoft.Office.Interop.Word.WdLineStyle.wdLineStyleDouble;

    //                oTable.AllowAutoFit = true;
    //                oTable.Columns.AutoFit();

    //            }

    //            string FilePath = System.IO.Path.GetTempPath() + "\\" + DateTime.Now.Second.ToString() + DateTime.Now.Millisecond.ToString() + ".pdf";
    //            oDoc.ExportAsFixedFormat(FilePath, Microsoft.Office.Interop.Word.WdExportFormat.wdExportFormatPDF, true);
    //            ((Microsoft.Office.Interop.Word._Document)(oDoc)).Close(Microsoft.Office.Interop.Word.WdSaveOptions.wdDoNotSaveChanges);
    //            ((Microsoft.Office.Interop.Word._Application)(oWord)).Quit(); 
    //        }
    //    }
    //    catch
    //    {
    //    }
    //}

    public void ExportToPDF(string Type, string Head, System.Windows.Forms.DataGridView Grid, System.Windows.Forms.DataGridViewRow GridRow)
    {
        try
        {
            DataTable dt = default(DataTable);
            if (Type == "One")
            {
                dt = DataGridViewRow_To_DataTable(Grid, GridRow, Head);
            }
            else
            {
                dt = DataGridView_To_DataTable(Grid, Head);
            }
            PDFform pdfForm = new PDFform(dt, Head, Type);

            Document document = pdfForm.CreateDocument();
            document.UseCmykColor = true;
            PdfDocumentRenderer pdfRenderer = new PdfDocumentRenderer(true);
            pdfRenderer.Document = document;
            pdfRenderer.RenderDocument();

            string FilePath = System.IO.Path.GetTempPath() + "\\" + DateTime.Now.Second.ToString() + DateTime.Now.Millisecond.ToString() + ".pdf";
            pdfRenderer.Save(FilePath);
            System.Diagnostics.Process.Start(FilePath);
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex.Message);
            Console.ReadLine();
        }
    }

    //Converts the DataGridView to DataTable
    public DataTable DataGridView_To_DataTable(System.Windows.Forms.DataGridView dgv, String tblName, int minRow = 0)
    {
        DataTable dt = new DataTable(tblName);

        // Header columns
        foreach (System.Windows.Forms.DataGridViewColumn column in dgv.Columns)
        {
            DataColumn dc = new DataColumn(column.Name.ToString());
            dt.Columns.Add(dc);
        }

        // Data cells
        for (int i = 0; i <= dgv.Rows.Count - 1; i++)
        {
            System.Windows.Forms.DataGridViewRow row = dgv.Rows[i];
            DataRow dr = dt.NewRow();
            for (int j = 0; j <= dgv.Columns.Count - 1; j++)
            {
                dr[j] = (row.Cells[j].Value == null) ? "" : row.Cells[j].Value.ToString();
            }
            dt.Rows.Add(dr);
        }

        // Related to the bug arround min size when using ExcelLibrary for export
        for (int i = dgv.Rows.Count; i <= minRow - 1; i++)
        {
            DataRow dr = dt.NewRow();
            for (int j = 0; j <= dt.Columns.Count - 1; j++)
            {
                dr[j] = "  ";
            }
            dt.Rows.Add(dr);
        }
        return dt;
    }

    //Converts the DataGridViewRow to DataTable
    public DataTable DataGridViewRow_To_DataTable(System.Windows.Forms.DataGridView dgv, System.Windows.Forms.DataGridViewRow dgvr, String tblName, int minRow = 0)
    {
        DataTable dt = new DataTable(tblName);

        // Header columns
        DataColumn dc1 = new DataColumn("Column");
        dt.Columns.Add(dc1);
        DataColumn dc2 = new DataColumn("Value");
        dt.Columns.Add(dc2);

        foreach (System.Windows.Forms.DataGridViewColumn column in dgv.Columns)
        {
            dt.Rows.Add(dgv.Columns[column.Index].Name.ToString(), dgvr.Cells[column.Index].Value);
        }
        return dt;
    }

}
